# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#

from unittest import TestCase
from facts.myredis import myredis
from facts.mylist import mylist

__author__ = 'fla'

""" Class to test the interaction with redis
"""

serverid = ""
tenantid = ""


class TestRedis(TestCase):
    pass

    def testInsertListWithOneElement(self):
        """testInsertOneElement should always return [] due to we insert a list with no elements"""
        p = myredis()

        expectedvalue = []
        p.insert(serverid, tenantid, [1])
        result = p.range(serverid, tenantid)

        self.assertEqual(expectedvalue, result)

    def testInsertListWithThreeElements(self):
        """testInsertOneElement should always return [] due to we insert a list with one element"""
        p = myredis()

        expectedvalue = ["['', 1, 2, 3]"]
        p.insert(serverid, tenantid, [serverid, 1, 2, 3])
        result = p.range(serverid, tenantid)

        self.assertEqual(expectedvalue, result)

    def testInsertListWithMoreThanThreeElements(self):
        """testInsertOneElement should always return [] due to we insert a list with more than 3 elements"""
        p = myredis()

        expectedvalue = []
        p.insert(serverid, tenantid, [serverid, 1, 2, 3, 4])
        result = p.range(serverid, tenantid)

        self.assertEqual(expectedvalue, result)

    def testInsertTwoCorrectElements(self):
        """testInsertTwoElement should always return two element in the list"""
        p = myredis()

        expectedvalue = ["['', 1, 2, 3]", "['', 5, 6, 7]"]
        p.insert(serverid, tenantid, [serverid, 1, 2, 3])
        p.insert(serverid, tenantid, [serverid, 5, 6, 7])
        result = p.range(serverid, tenantid)

        self.assertEqual(expectedvalue, result)

    def testInsertGTFiveElement(self):
        """testInsertGTFiveElement should always return five element if we have
        five or more than five element in the list"""
        p = myredis()

        expectedvalue = ["['', 5, 6, '8']", "['', 9, 10, '12']", "['', 13, 14, '16']",
                         "['', 17, 18, '20']", "['', 21, 22, '24']"]

        p.insert(serverid, tenantid, [serverid, 1, 2, '4'])
        p.insert(serverid, tenantid, [serverid, 5, 6, '8'])
        p.insert(serverid, tenantid, [serverid, 9, 10, '12'])
        p.insert(serverid, tenantid, [serverid, 13, 14, '16'])
        p.insert(serverid, tenantid, [serverid, 17, 18, '20'])
        p.insert(serverid, tenantid, [serverid, 21, 22, '24'])
        result = p.range(serverid, tenantid)

        self.assertEqual(expectedvalue, result)

    def testSumOneValue(self):
        """testSumValores should return the sum of a list
         of values"""
        p = myredis()
        expected = [["''", 1.0, 2.0, '3']]
        p.insert(serverid, tenantid, [serverid, 1, 2, 3])
        result = p.sum(p.range(serverid, tenantid))

        self.assertEqual(expected, result.data)

    def testSumZeroValue(self):
        """testSumValores should return the sum of a list
         of values"""
        p = myredis()
        expected = '[]'
        result = p.sum(p.range(serverid, tenantid))

        self.assertEqual(expected, result)

    def testSumListValue(self):
        """testSumValores should return the sum of a list
         of values"""
        p = myredis()

        expected = ["''", 4, 8, '3']

        p.insert(serverid, tenantid, [serverid, 1, 2, 3])
        p.insert(serverid, tenantid, [serverid, 1, 2, 3])
        p.insert(serverid, tenantid, [serverid, 1, 2, 3])
        p.insert(serverid, tenantid, [serverid, 1, 2, 3])

        li = p.range(serverid, tenantid)

        result = p.sum(li)

        self.assertEqual(expected, result.data)

    def testSumListValue2(self):
        """testSumValores should return the sum of the last
        5 values of the list of values"""
        p = myredis()

        expected = ["''", 1111100, 2222200, '3000000']

        p.insert(serverid, tenantid, [serverid, 1, 2, '4'])
        p.insert(serverid, tenantid, [serverid, 10, 20, '40'])

        p.insert(serverid, tenantid, [serverid, 100,         200,     300])
        p.insert(serverid, tenantid, [serverid, 1000,       2000,    3000])
        p.insert(serverid, tenantid, [serverid, 10000,     20000,   30000])
        p.insert(serverid, tenantid, [serverid, 100000,   200000,  300000])
        p.insert(serverid, tenantid, [serverid, 1000000, 2000000, 3000000])

        li = p.range(serverid, tenantid)

        result = p.sum(li)

        self.assertEqual(expected, result.data)

    def testMediaListof4Values(self):
        """ return the media of a list of 4 values
        """
        p = myredis()

        expected = ["''", 6, 7, '14']

        p.insert(serverid, tenantid, [serverid, 0, 1, 2])
        p.insert(serverid, tenantid, [serverid, 3, 4, 5])
        p.insert(serverid, tenantid, [serverid, 6, 7, 8])
        p.insert(serverid, tenantid, [serverid, 9, 10, 11])
        p.insert(serverid, tenantid, [serverid, 12, 13, 14])

        li = p.range(serverid, tenantid)

        result = p.media(li)

        self.assertEqual(expected, result.data)

    def testRealData(self):
        """Test with real data"""
        p = myredis()

        p1 = "[1.0, 0.14, '2014-03-29T19:18:25.784424']"

        expected = []

        p2 = mylist.parselist(p1)

        p.insert(serverid, tenantid, p2)
        p.insert(serverid, tenantid, p2)

        result = p.media(p.range(serverid, tenantid))

        self.assertEqual(expected, result.data)

    def testRealData2(self):
        """Test with real data"""
        p = myredis()

        p1 = "[, 1.0, 0.14, '2014-03-29T19:18:25.784424']"

        expected = ["''", 1.0, 0.14, '2014-03-29T19:18:25.784424']

        p2 = mylist.parselist(p1)

        p.insert(serverid, tenantid, p2)
        p.insert(serverid, tenantid, p2)
        p.insert(serverid, tenantid, p2)
        p.insert(serverid, tenantid, p2)
        p.insert(serverid, tenantid, p2)

        result = p.media(p.range(serverid, tenantid))

        self.assertEqual(expected, result.data)

    def testCheckTimeStamp(self):
        """Test if the time stamp of the new element is valid comparing to the last element."""
        p = myredis()

        p1 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:25.784424']"
        p2 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:30.784424']"
        p3 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:35.784424']"
        p4 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:40.784424']"
        p5 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:45.784424']"

        expected = True

        p.insert(serverid, tenantid, mylist.parselist(p1))
        p.insert(serverid, tenantid, mylist.parselist(p2))
        p.insert(serverid, tenantid, mylist.parselist(p3))
        p.insert(serverid, tenantid, mylist.parselist(p4))

        result = p.check_time_stamps(tenantid, serverid, p.range(serverid, tenantid), p5)

        self.assertEqual(expected, result)

    def testCheckTimeStampInvalid(self):
        """Test if the time stamp of the new element is invalid comparing to the last element."""
        p = myredis()

        p1 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:25.784424']"
        p2 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:30.784424']"
        p3 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:35.784424']"
        p4 = "['serverId', 1.0, 0.14, '2014-03-29T19:18:40.784424']"
        p5 = "['serverId', 1.0, 0.14, '2014-03-30T19:18:45.784424']"

        expected = False

        p.insert(serverid, tenantid, mylist.parselist(p1))
        p.insert(serverid, tenantid, mylist.parselist(p2))
        p.insert(serverid, tenantid, mylist.parselist(p3))
        p.insert(serverid, tenantid, mylist.parselist(p4))

        result = p.check_time_stamps(tenantid, serverid, p.range(serverid, tenantid), p5)

        self.assertEqual(expected, result)



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
__author__ = 'fla'

from unittest import TestCase
from facts.jsoncheck import jsoncheck

mydict1 = {
    "contextResponses": [
        {
            "contextElement": {
                "attributes": [
                    {
                        "contextValue": "6",
                        "name": "users",
                        "type": "string"
                    },
                    {
                        "contextValue": "1",
                        "name": "usedMemPct",
                        "type": "string"
                    },
                    {
                        "contextValue": "0.14",
                        "name": "cpuLoadPct",
                        "type": "string"
                    },
                    {
                        "contextValue": "0.856240",
                        "name": "freeSpacePct",
                        "type": "string"
                    }
                ],
                "id": "Trento:193.205.211.69",
                "isPattern": "false",
                "type": "host"
            },
            "statusCode": {
                "code": "200",
                "reasonPhrase": "OK"
            }
        }
    ]
}


mydict2 = {"uno": 1, "dos": "dos", "tres": "tres"}

mydict3 = {"contextResponses": 1}

mydict4 = {"a": 1}


class Testjsoncheck(TestCase):
    pass

    def test_wellformedmessage(self):
        """ Check that a correct message is recognize
        """
        key = ['contextResponses', 'contextElement', 'attributes']

        expectedvalue = ""

        try:
            jsoncheck.checkit(mydict1, key, 0)
        except (Exception), err:
            self.assertEqual(expectedvalue, err.message)

    def test_unknowkeyvalue(self):
        """ Check that not all keys are content in the dict¡onary
        """
        key = ['contextResponses', 'context', 'attributes']

        expectedvalue = "Invalid json message. We cannot obtain the key: context"

        try:
            jsoncheck.checkit(mydict1, key, 0)
        except (Exception), err:
            self.assertEqual(expectedvalue, err.message)

    def test_unknowjsonmessage(self):
        """ Check that a not expected message is recognized
        """
        key = ['contextResponses', 'contextElement', 'attributes']

        expectedvalue = "Invalid json message. We cannot obtain the key: contextResponses"

        try:
            jsoncheck.checkit(mydict2, key, 0)
        except (Exception), err:
            self.assertEqual(expectedvalue, err.message)

    def test_dictwithonekey(self):
        """Check that an invalid json with only one valid key is recognized
        """
        key = ['contextResponses', 'contextElement', 'attributes']

        expectedvalue = "Invalid json message. We expected " \
                        "'['contextResponses', 'context', 'attributes']' keys but " \
                        "obtain only 'contextResponses' key"

        try:
            jsoncheck.checkit(mydict3, key, 0)
        except (Exception), err:
            self.assertEqual(expectedvalue, err.message)

    def test_dictwithoneinvalidkey(self):
        """Check that an invalid json with only one valid key is recognized
        """
        key = ['contextResponses', 'contextElement', 'attributes']

        expectedvalue = "Invalid json message. We cannot obtain the key: contextResponses"

        try:
            jsoncheck.checkit(mydict4, key, 0)
        except (Exception), err:
            self.assertEqual(expectedvalue, err.message)



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#

__version__ = '1.2.0'
__version_info__ = tuple([int(num) for num in __version__.split('.')])
__description__ = 'Facts Listener'
__author__ = 'fla'

from flask import Flask, request, Response, json
from facts.myredis import myredis
from facts.queue import myqueue
from facts.jsoncheck import jsoncheck
from gevent.pywsgi import WSGIServer
import logging.config
import sys
import datetime
import gevent.monkey
import os
import httplib


gevent.monkey.patch_all()

content_type = 'application/json'

from facts.config import config, cfg_filename, cfg_defaults

"""Flask server initialization.

Uses Redis server as a message queue and server exchange with the fiware-cloto.
"""
app = Flask(__name__)

"""
Initialize the redis connection library
"""
mredis = myredis()

"""
Initialize the pid of the process
"""
pid = 0

# Flask/Gevent server need to send {'serverId': 'serverId', 'cpu': 80, 'mem': 80, 'time': '2014-03-24 16:21:29.384631'}
# to the topic


@app.route('/v1.0', methods=['GET'])
def factsinfo():
    """API endpoint for receiving keep alive information
    """
    return Response(response="{\"fiware-facts\":\"Up and running...\"}\n",
                    status=httplib.OK,
                    content_type=content_type)


@app.route('/v1.0/<tenantid>/servers/<serverid>', methods=['POST'])
def facts(tenantid, serverid):
    """API endpoint for receiving data from Monitoring system

    :param string tenantid:    the id of the tenant
    :param string serverid:    the id of the monitored instance (server)

    :return: status code 405 - invalid JSON or invalid request type
    :return: status code 400 - unsupported Content-Type or invalid publisher
    :return: status code 200 - successful submission
    """
    # Ensure post's Content-Type is supported
    if request.headers['content-type'] == content_type:
        try:
            # Ensure that received data is a valid JSON
            user_submission = json.loads(request.data)  # @UnusedVariable
        except ValueError:
            # Data is not a well-formed json
            message = "[{}] received {} from ip {}:{}"\
                .format("-", json, request.environ['REMOTE_ADDR'], request.environ['REMOTE_PORT'])

            logging.warning(message)

            return Response(response="{\"error\":\"Bad request. The payload is not well-defined json format\"}\n",
                            status=httplib.BAD_REQUEST,
                            content_type=content_type)

        # It is a valid payload and we start to process it
        result = process_request(request, tenantid, serverid)

        if result == True:
            return Response(status=httplib.OK)
        else:
            return Response(response="{\"error\":\"Internal Server Error. Unable to contact with RabbitMQ process\"}\n",
                            status=httplib.INTERNAL_SERVER_ERROR,
                            content_type=content_type)

    # User submitted an unsupported Content-Type (only is valid application/json)
    else:
        return Response(response="{\"error\":\"Bad request. Content-type is not application/json\"}\n",
                        status=httplib.BAD_REQUEST,
                        content_type=content_type)


def process_request(request, tenantid, serverid):
    """Get the parsed contents of the form data

    :param string request:     The information of the received request
    :param string serverid:    the id of the monitored instance (server)

    :return: True
    """
    json = request.json
    message = "[{}] received {} from ip {}:{}"\
        .format("-", json, request.environ['REMOTE_ADDR'], request.environ['REMOTE_PORT'])

    logging.info(message)

    key = ['contextResponses', 'contextElement', 'attributes']

    # Check that it contains the previous keys
    try:
        jsoncheck.checkit(json, key, 0)
    except (Exception), err:
        logging.error(err)
        return Falsechmod

    # Extract the list of attributes from the NGSI message
    attrlist = request.json['contextResponses'][0]['contextElement']['attributes']

    data = list()

    for item in attrlist:
        name = item['name']
        value = item['contextValue']

        # Obtain the information of used memory and cpu
        if name == 'usedMemPct' or name == 'cpuLoadPct':
            data.insert(len(data), float(value))

    # fix the first value of the list with the server identity
    data.insert(0, str(serverid))

    # fix the last value with the current date and time
    data.insert(3, datetime.datetime.today().isoformat())

    # Check data coherency of time stamps
    if len(mredis.range(tenantid, serverid)) > 2:
        mredis.check_time_stamps(tenantid, serverid, mredis.range(tenantid, serverid), data)

    # Insert the result into the queue system
    mredis.insert(tenantid, serverid, data)
    logging.info(data)

    # If the queue has the number of facts defined by the windows size, it returns the
    # last window-size values (range) and calculates the media of them (in terms of memory and cpu)
    lo = mredis.media(mredis.range(tenantid, serverid))

    # If the number of facts is lt window size, the previous operation returns a null lists
    if len(lo) != 0:
        try:
            rabbit = myqueue()

            message = "{\"serverId\": \"%s\", \"cpu\": %d, \"mem\": %d, \"time\": \"%s\"}" \
                      % (lo.data[0], lo.data[1], lo.data[2], lo.data[3])

            logging_message = "[{}] sending message {}".format("-", message)

            logging.info(logging_message)

            # Send the message to the RabbitMQ components.
            result = rabbit.publish_message(tenantid, message)  # @UnusedVariable

        except Exception:
            #logging.info(lo.get())
            return False

    return True


def info(port):
    """Show some information about the execution of the process.
    """

    data = config.get('common', 'name')
    pid = os.getpid()

    logging.info("{} {}\n".format(data, __version__))
    logging.info("Running in stand alone mode")
    logging.info("Port: {}".format(port))
    logging.info("PID: {}\n".format(pid))
    logging.info("https://github.hi.inet/telefonicaid/fiware-facts\n\n\n")


# process configuration file (if exists) and setup logging
if config.read(cfg_filename):
    logging.config.fileConfig(cfg_filename)
else:
    logging.basicConfig(stream=sys.stdout, level=cfg_defaults['logLevel'], format=cfg_defaults['logFormat'])

# Define the port of our server, by default 5000
port = config.getint('common', 'brokerPort')

# execute the flask server, WSGI server
http = WSGIServer(('', port), app)

# show general information about the execution of the process
info(port)

http.serve_forever()



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefónica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#

__author__ = 'fla'

from ConfigParser import SafeConfigParser
import os.path
import datetime


"""
Default configuration.

The configuration `cfg_defaults` can be superseded with that read from `cfg_filename`
(at path `../conf/<progname>.cfg`), if file exists.
"""

name = 'fiware-facts'
cfg_filename = os.path.join(os.path.dirname(__file__), '..', 'conf', '%s.cfg' % name)
cfg_defaults = {
    'brokerPort':   5000,                   # port of our facts broker
    'redisPort':    6379,                   # port of Redis
    'redisHost':    'localhost',            # host of Redis
    'redisQueue':   'policymanager',        # name of the queue in redis
    'rabbitMQ':     'localhost',        # IP of the RabbitMQ server
    'name':         'policymanager.facts',  # name of the server
    'logLevel':     'INFO',
    'logFormat':    '%(asctime)s %(levelname)s policymanager.facts %(message)s'
}

config = SafeConfigParser(cfg_defaults)

# Create the common section in the same way that we have in the configuration file: fiware-facts.cfg
config.add_section('common')

# Initialize the content of the config parameters
for key, value in cfg_defaults.items():
    config.set('common', key, str(value))

windowsize_facts = datetime.timedelta(seconds=10)

fact_attributes = ['serverId', 'CpuValue', 'MemValue', 'DataTime']



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
__author__ = 'fla'

from flask.ext.testing import TestCase
from flask import Flask
import urllib2
import json

"""Class to test the flask, gevent process
"""


class MyTest(TestCase):
    def create_app(self):
        app = Flask(__name__)
        app.config['TESTING'] = True
        self.url = 'http://127.0.0.1:5000/v1.0/33/servers/44'
        self.url2 = 'http://127.0.0.1:5000/v1.0'
        return app

    def test_server_is_up_and_running(self):
        """ Test that a fiware-facts is up and running and return
        information of the GET operation

        :return       200 Ok
        """
        response = urllib2.urlopen(self.url2)
        self.assertEqual(response.code, 200)

    def test_some_json(self):
        """ Test that the POST operation over the API returns a error if
        content-type is not application/json.

        :return       500 Ok
        """
        data = {'ids': [12, 3, 4, 5, 6]}

        req = urllib2.Request(self.url)

        req.add_header('Content-Type', 'application/json')

        try:
            response = urllib2.urlopen(req, json.dumps(data))

        except (urllib2.HTTPError), err:
            self.assertEqual(err.code, 500)
            self.assertEqual(err.msg, "INTERNAL SERVER ERROR")



__author__ = 'fla'

#!flask/bin/python
from werkzeug.contrib.profiler import ProfilerMiddleware
from flask import Flask

app = Flask(__name__)

app.config['PROFILE'] = True
app.wsgi_app = ProfilerMiddleware(app.wsgi_app, restrictions=[30])
app.run(debug=True)



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'

from config import config, fact_attributes
from mylist import mylist
from redis.exceptions import ConnectionError
import logging
import redis

nqueue = config.get('common', 'redisQueue')

# TODO: change the default queue in redis to incorporate the server ID.
# TODO: Check the temporal mark when we want to calculate the media of data.
# TODO: Review Kalman filter in order to change the estimation of the data in the windows size.


class myredis(object):
    """
    Class to create lists on redis distribution. it is assumed
    that a redis-server is running at host:port.
    """
    def __init__(self):
        """ Initialize the class and create a connection with a redis server instance.
        The initialization includes the deletion of the previous queue
        """
        host = config.get('common', 'redisHost')
        port = config.getint('common', 'redisPort')
        self.r = redis.StrictRedis(host=host, port=port, db=0)

        try:
            # TODO: We need to delete all the queues.
            self.r.flushall()
            #self.r.delete(nqueue)

        except ConnectionError:
            message = "[{}] Cannot delete the list. Possibly redis is down".format("-")
            logging.error(message)

    def insert(self, tenantid, serverid, data):
        """ Insert data into the redis queue.

        :param list data:     The list of data to be stored in the queue
        :return               This operation does not return anything except when the data
                              is no list or the number of element is not equal to 4.
        """
        if isinstance(data, list) and len(data) == len(fact_attributes):
            self.r.rpush(tenantid + "." + serverid, data)
            self.r.ltrim(tenantid + "." + serverid, -5, -1)
        else:
            return "error"

    def range(self, tenantid, serverid):
        """ Return the list of element stored the que queue.
         :return a list of lists
        """
        return self.r.lrange(tenantid + "." + serverid, -100, 100)

    def media(self, lista):
        """ Calculate the media of a list of lidts

         :param mylist lista     The mylist instance with the data to be added.
         :return mylist          The media of the data
        """
        if len(lista) >= 5:
            return self.sum(lista) / len(lista)
        else:
            result = mylist()
            result.data = []
            return result

    def sum(self, lista):
        """ Calculate the sum of a list of data

        :param mylist lista:     The lists of lists which will be added.
        return mylist            The new mylist instance with the result of the operation.
        """
        if len(lista) > 1:
            return mylist.sum(lista)
        elif len(lista) == 1:
            r1 = mylist()
            r1.insert(lista)
            return r1
        else:
            return '[]'

    def delete(self):
        """ Delete a especific queue from the redis system.
        """
        self.r.delete(nqueue)

    def check_time_stamps(self, tenantid, serverid, lista, data):
        """
        Check if the list is valid checking last item time-stamp with the new item time-stamp
        :return:
        """
        from dateutil import parser
        textmin = lista[-1].split("'")
        textmax = data.split("'")
        datemin = parser.parse(textmin[-2], fuzzy=True)
        datemax = parser.parse(textmax[-2], fuzzy=True)
        from config import windowsize_facts

        timediff = datemax - datemin
        if  timediff > windowsize_facts:
            self.r.delete(tenantid + "." + serverid)
            return False
        else:
            return True



# -*- encoding: utf-8 -*-
#
# Copyright 2014 Telefonica Investigación y Desarrollo, S.A.U
#
# This file is part of FI-WARE project.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
#
# You may obtain a copy of the License at:
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#
# See the License for the specific language governing permissions and
# limitations under the License.
#
# For those usages not covered by the Apache version 2.0 License please
# contact with opensource@tid.es
#
__author__ = 'fla'

from unittest import TestCase
from facts.mylist import mylist


class Testmylist(TestCase):
    pass

    def testisdatatrue(self):
        """testisdatatrue should always return true due to p1 is mylist and the length is 3"""
        p1 = mylist()

        p1.insert([2, 3, 4])

        expectedvalue = True

        result = mylist.isdata(p1)

        self.assertEqual(expectedvalue, result)

    def testisdatafalse1(self):
        """testisdatafalse should always return false due to p1 is mylist but the length is not equal to 3"""
        p1 = mylist()

        expectedvalue = False

        p1.insert([1, 2])

        result = mylist.isdata(p1)

        self.assertEqual(expectedvalue, result)

    def testisdatafalse5(self):
        """testisdatafalse should always return false due to p1 is mylist but the length is not equal to 3"""
        p2 = mylist()

        expectedvalue = False

        p2.insert([1, 2, 3, 4])

        result = mylist.isdata(p2)

        self.assertEqual(expectedvalue, result)

    def testisdatafalse2(self):
        """testisdata should always return false due to p1 is mylist but the content is not a list"""
        p1 = mylist()

        p1.insert(1)

        expectedvalue = False

        result = mylist.isdata(p1)

        self.assertEqual(expectedvalue, result)

    def testisdatafalse3(self):
        """testisdata should always return false due to p1 is not mylist"""
        p1 = 1

        expectedvalue = False

        result = mylist.isdata(p1)

        self.assertEqual(expectedvalue, result)

    def testinit(self):
        """ check the creation of a instance
        """
        p1 = mylist([2, 3, 4])

        expectedvalue = [2, 3, 4]

        result = p1.get()

        self.assertEqual(expectedvalue, result)

    def testsum(self):
        """check the sum of list"""
        p1 = [['serverId', 1, 2, 3], ['serverId', 1, 2, 4], ['serverId', 1, 2, 5]]

        expectedvalue = ['serverId', 3, 6, 5]

        result = mylist.sum(p1)

        self.assertEqual(expectedvalue, result.data)

    def testdivlist(self):
        """check the division by a integer the list except the first one"""
        p1 = ['serverId', 2, 4, 8]

        expectedvalue = ['serverId', 1, 2, 8]

        p1mylist = mylist(p1)

        result = p1mylist / 2

        self.assertEqual(expectedvalue, result.data)

    def testdivlistFloat(self):
        """check the division by a float the list except the first one"""
        p1 = ['serverId', 3, 5, 8]

        expectedvalue = ['serverId', 1.5, 2.5, 8]

        p1mylist = mylist(p1)

        result = p1mylist / 2

        self.assertEqual(expectedvalue, result.data)

    def testmedia(self):
        """check the media of a list of data"""

        expectedvalue = ['serverId', 7, 8, 18]

        p1 = [['serverId', 1, 2, 3], ['serverId', 5, 6, 7], ['serverId', 9, 10, 11], ['serverId', 13, 14, 18]]

        result = mylist.sum(p1) / len(p1)

        self.assertEqual(expectedvalue, result.data)

    def testsum2(self):
        """check the sum of a list of strings but with commas"""
        p1 = ['[serverId, 1, 2, 1]', '[serverId, 1, 2, 2]', '[serverId, 1, 2, 3]', '[serverId, 1, 2, 4]']

        expectedvalue = ['serverId', 4.0, 8.0, '4']

        result = mylist.sum(p1)

        self.assertEqual(expectedvalue, result.data)

    def testinsert(self):
        """check the insertion of list of strings"""
        p1 = ['[serverId, 1, 2, 4]']

        r1 = mylist()

        expectedvalue = [['serverId', 1, 2, '4']]

        r1.insert(p1)

        result = r1.get()

        self.assertEqual(expectedvalue, result)

    def testinsert2(self):
        """check the insertion of list of strings"""
        p1 = ['[serverId, 1, 2, 4]', '[serverId, 1, 2, 4]', '[serverId, 1, 2, 4]', '[serverId, 1, 2, 4]']

        r1 = mylist()

        expectedvalue = [['serverId', 1.0, 2.0, '4'], ['serverId', 1.0, 2.0, '4'], ['serverId', 1.0, 2.0, '4'], ['serverId', 1.0, 2.0, '4']]

        r1.insert(p1)

        result = r1.get()

        self.assertEqual(expectedvalue, result)

    def testRealInsert(self):
        """test insertion of real data"""

        expectedvalue = [['serverId', 1.0, 0.14, '2014-03-29T19:18:25.784424']]
        p1 = ["[serverId, 1.0, 0.14, '2014-03-29T19:18:25.784424']"]

        r1 = mylist()
        r1.insert(p1)

        result = r1.get()

        self.assertEqual(expectedvalue, result)

    def testRealInsert2(self):
        """test insertion of real data"""

        expectedvalue = ['serverId', 1.0, 0.14, '2014-03-29T19:18:25.784424']
        p1 = "[serverId, 1.0, 0.14, '2014-03-29T19:18:25.784424']"

        r1 = mylist()
        r1.insert(mylist.parselist(p1))

        result = r1.get()

        self.assertEqual(expectedvalue, result)

    def testReadMedia1Data(self):
        """test the insertion of a real data and calculate its media"""

        fact = ['serverId', 1.0, 0.14, '2014-03-29T23:02:46.973949']
        expectedvalue = ['serverId', 1.0, 0.14, '2014-03-29T23:02:46.973949']

        p2 = list()

        p2.insert(0, fact)
        p2.insert(0, fact)

        result = mylist.sum(p2)

        result = result / len(p2)

        self.assertEqual(expectedvalue, result.data)
